import java.util.StringTokenizer;

/** 
 * Description:
 * Je nach der über den Konstruktor bestimmten Position – ganz am
 * Anfang oder Ende des Programms, oder vor oder nach jeder Zeile,
 * die eine bestimmte Zeichenfolge (außerhalb eines Kommentars bzw.
 * Strings) enthält – wird ein sich dort befindlicher Kommentar entfernt.
 * Steht an dieser Stelle kein Kommentar, bleibt das Programm
 * unverändert.
 * 
 * @author Robert Kapeller (e8727334)
 *
 */
public class Eraser implements Prettifier {

	// constants used in the program
	private final String COMMENT_REG_EXP = "(?:/\\*(?:[^*]|(?:\\*+[^*/]))*\\*+/)|(?://.*)";
	private final String STAR_COMMENT_START = "/*";
	private final String STAR_COMMENT_END = "*/";
	private final String SLASH_COMMENT = "//";
	private final String NEW_LINE = "\n";

	// pattern to be searched for in code
	private String pattern;
	// where to remove the comments
	private PrettyPos position;	
		
	/**
	 * VB:	pattern and position must be a useful combination:
	 * 		if pattern != null: position in (PrettyPos.BEFORE, PrettyPos.AFTER)
	 * 		if position is anything else, pattern will be ignored. 
	 * NB:  Sets the private paramters to be used for "pretty"
	 * 
	 * Erases a comment after or before each line containing the pattern or at the end of the file or at the beginning 
	 * or any comments depending on the 2nd paramter. 'pattern' is ignored if the second paramter is anything else than
	 * PrettyPos.Before or PrettyPos.After,<br>
	 * Useful Position in conjunction with a String != null is PrettyPos.BEFORE or PrettyPos.AFTER
	 * 
	 * @param pattern
	 * @param position
	 */
	public Eraser (String pattern, PrettyPos position) {
		this.pattern = pattern;
		this.position = position;
	}
		
	/**
	 * VB: prog != null
	 * NB: returns changed prog wit removed comments according to the settings of the constructor. prog is unchanged
	 *
	 * @param prog
	 * @return

	 */
	@Override
	public String pretty(String prog) {
		switch (position) {
			case ANY:
				return removeAll(prog);
			case BEFORE:
				return removeBefore(prog);
			case AFTER:
				return removeAfter(prog);
			case BEGIN:
				return removeBeginning(prog);
			case END:		
				return removeEnd(prog);
		}
		return null;
	}
	
	/**
	 * VB: prog != null. A valid prog starts with one of: 'public', 'private', 'class', 'enum', 'final'
	 * NB: returns prog without comments at the beginning of the String (if there is any), prog iteself is unchanged.
	 * 
	 * @param prog
	 * @return
	 */
	private String removeBeginning(String prog) {

		StringBuilder s = new StringBuilder();
		
		String[] parts = prog.split(NEW_LINE);
		boolean isComment = true;
		for (String line: parts) {
			// jump over all entries where there is no comment in it
			if (isComment) {
				if (line.matches(".*public.*|.*private.*|.*class.*|.*enum.*|.*final.*")) {
					isComment = false;
					s.append(line).append(NEW_LINE);
				}
			} else {
				s.append(line).append(NEW_LINE);
			}
		}
		return s.toString();
	}
	
	/**
	 * VB: prog != null. The end of the program is assumed to be everything after the last '}'
	 * NB: returns prog without comments at the end of the String (if there is any). Returns the changed program, prog iteself is unchanged.
	 * 
	 * @param prog
	 * @return
	 */
	private String removeEnd(String prog) {
		StringBuilder s = new StringBuilder();
		
		String[] parts = prog.split(NEW_LINE);
		boolean isBracket = false;
		for (int i = parts.length-1; i >= 0; i--) {
			// jump over all entries where there is no comment in it
			if (!isBracket) {
				if (parts[i].matches(".*}.*")) {
					isBracket = true;
					s.insert(0, parts[i] + NEW_LINE);
				}
			} else {
				s.insert(0, parts[i] + NEW_LINE);
			}
		}
		return s.toString();
	}

	/**
	 * VB: prog != null
	 * NB: returns prog without comments before each occurence of the pattern. Returns the changed program, prog iteself is unchanged.
	 * 
	 * @param prog
	 * @return
	 */
	private String removeBefore(String prog) {
		StringBuilder sb = new StringBuilder();
		String[] parts = prog.split(NEW_LINE);

		// Suche nach Zeilen, in denen das pattern vorkommt
		for (int i = parts.length - 1; i >= 0 ; i--) {
			if (parts[i].contains(pattern)) {
				sb.insert(0, parts[i] + NEW_LINE);
				if ( i > 0 && isComment(parts[i-1])) {
					i = i - 1; // Jump over comment, do not add it it to output string
				}
			} else {
				sb.insert(0, parts[i] + NEW_LINE);
			}
		}
		return sb.toString();
	}

	/**
	 * VB: prog != null
	 * NB: returns prog without comments after each occurence of the pattern. Returns the changed program, prog iteself is unchanged.
	 * 
	 * @param prog
	 * @return
	 */
	private String removeAfter(String prog) {
		StringBuilder sb = new StringBuilder();
		String[] parts = prog.split(NEW_LINE);
		int linecount = parts.length;
		// Suche nach Zeilen, in denen das pattern vorkommt
		for (int i = 0; i < linecount; i++) {
			if (parts[i].contains(pattern)) {
				sb.append(parts[i] + NEW_LINE);					
				if (i < linecount-1 && isComment(parts[i+1])) {
					i = i + 1; // Jump over comment, do not add it it to output string
				}
			} else {
				sb.append(parts[i] + NEW_LINE);
			}
		}
		return sb.toString();
	}
	
	/**
	 * VB: 	s != null
	 * NB: 	returns true if s is a comment-line (it eather starts with slash-star and ends with star-slash (without whitespaces)<br>
	 * or it starts with // (without whitespaces))
	 * 		false otherwise
	 * 
	 * 
	 * @param s
	 * @return
	 */
	private boolean isComment(String s) {
		String toCheck = s.trim();
		if (toCheck.startsWith(SLASH_COMMENT))
			return true;
		if (toCheck.startsWith(STAR_COMMENT_START) && toCheck.endsWith(STAR_COMMENT_END))
				return true;
		return false;
	}
	/**
	 * VB: 	prog != null
	 * NB: returns prog without comments. Returns the changed program, prog iteself is unchanged.
	 *
	 * @param prog
	 * @return
	 */
	private String removeAll(String prog) {
		return prog.replaceAll(COMMENT_REG_EXP,"");
	}
}
